#include "PHAffineTransform.h"
#include <Foundation/NSException.h>
#include "PHGeometry.h"

@implementation NSAffineTransform (PhotoClip)

+ (NSAffineTransform *) transformToQIofSize: (NSSize) size
                                      scale: (float) scale
                                   rotation: (float) degrees
{
    NSAffineTransform * xform = [NSAffineTransform transform];

    [xform scaleBy: scale];
    [xform rotateByDegrees: degrees];

    NSSize rotated_size = [xform transformSize: size];
    
    // get coordinates of the origin
    float x=0, y=0;

    switch ((int)degrees)
    {
        case 0:
            x = y = 0;
            break;
        case 180:
            x = rotated_size.width;
            y = rotated_size.height;
            break;
        case 90:
            x = rotated_size.width;
            y = 0;
            break;
        case 270:
            x = 0;
            y = rotated_size.height;
            break;
        default:
            NSAssert1(0, @"Invalid rotation angle=%f\n", degrees);
            break;
    }

    [xform translateXBy: x yBy: y];

    return xform;
}

+ (NSAffineTransform *) transformToQIofRect: (NSRect) rect
                                      scale: (float) scale
                                   rotation: (int) degrees
{
    NSAffineTransform * xform = [NSAffineTransform transform];
    [xform scaleBy: scale];
    [xform rotateByDegrees: degrees];

    NSRect transformed_rect = [xform transformRect: rect];


    // translate reference frame to map rectangle 'rect' into first quadrant
    [xform translateXBy: -transformed_rect.origin.x
                    yBy: -transformed_rect.origin.y];
}

- (NSRect) transformRect: (NSRect) rect
{
    NSPoint end = NSMakePoint(rect.origin.x + rect.size.width,
                              rect.origin.y + rect.size.height);
    NSPoint a = [self transformPoint: rect.origin];
    NSPoint b = [self transformPoint: end];

    return PHRectFromPoints(a, b);
}

@end
